# -*- coding: utf-8 -*- 
# @Time    : 03/21/2024 
# @Author  : nzooherd 
# @File    : video.py 
# @Software: Visual Studio Code


from telethon.events import filters, NewMessage, ButtonCallback
from telethon.types import Message

from config import Config
from aspect.api import ApiAspect
from controller import BaseController, CallbackPrefix, PornherdBot
from service.message_service import MessageService
from service.video_service import VideoService
from utills.bytes_util import CallbackBytesRequest, byte2int

class VideoController(BaseController):

    def __init__(self, mybot: PornherdBot) -> None:
        super().__init__(mybot)

        self.video_service = VideoService()
        self.message_service = MessageService()

    @ApiAspect(filters.All(filters.Command('/info'), filters.Chats(Config.TELEGRAM_ADMIN_USER_ID)))
    async def info(self, update: NewMessage) -> Message:
        """
        Get info of video
        """
        template = await self.video_service.get_video_message(update.text.split(" ")[1])
        message = await self.mybot.bot_client.send_message(update.chat, template=template, link_preview=True)
        return message

    @ApiAspect(filters.Command('/search'))
    async def search(self, update: NewMessage) -> Message:
        """
        """
        template = await self.video_service.search_video_init()
        await self.mybot.bot_client.send_message(update.chat, template=template)

    @ApiAspect(filter=CallbackPrefix("/tags_c"))
    async def tag_select_callback(self, update: ButtonCallback) -> Message:
        """
        """
        request: CallbackBytesRequest = CallbackBytesRequest.from_byte(["tag_id", "mark_tags"], [1, 8], update.data)
        message_id = request.message_id
        mark_tags = request.mark_tags
        tag_id = byte2int(request.tag_id)
        chat = list(update._chat_map.values())[0]

        template = await self.video_service.select_tag(mark_tags, tag_id)
        old_message = await self.message_service.select_by_id(message_id)

        await self.mybot.bot_client.edit_message(chat, old_message=old_message, template=template)

    @ApiAspect(filter=CallbackPrefix("/sv_c")) # serach_video_callback
    async def search_callback(self, update: ButtonCallback) -> Message:
        """
        """
        request: CallbackBytesRequest = CallbackBytesRequest.from_byte(["mark_tags"], [8], update.data)
        message_id = request.message_id
        mark_tags = request.mark_tags

        chat = list(update._chat_map.values())[0]
        template = await self.video_service.search_video_scroll(mark_tags)
        await self.mybot.bot_client.send_message(chat, template=template, link_preview=True, reply_to=message_id)

    @ApiAspect(filter=CallbackPrefix("/scs_c")) #search_video_scroll_callback
    async def search_scroll_callback(self, update: ButtonCallback) -> Message:
        """
        """
        request: CallbackBytesRequest = CallbackBytesRequest.from_byte(["mark_tags", "offset"], [8, 8], update.data)
        message_id = request.message_id
        offset = byte2int(request.offset)

        old_messge = await self.message_service.select_by_id(message_id)
 
        chat = list(update._chat_map.values())[0]
        template = await self.video_service.search_video_scroll(request.mark_tags, offset)
        await self.mybot.bot_client.edit_message(chat, old_message=old_messge, template=template, link_preview=True)

    @ApiAspect(filters.Command("/videos"))
    async def videos(self, update: NewMessage) -> Message:
        """
        """
        template = await self.video_service.videos_init()
        message = await self.mybot.bot_client.send_message(update.chat, template=template, link_preview=True)
        return message

    @ApiAspect(filter=CallbackPrefix("/vo_c"))
    async def videos_callback(self, update: ButtonCallback) -> Message:
        """
        视频列表回调
        """
        request: CallbackBytesRequest = CallbackBytesRequest.from_byte(["video_id", "scroll_dir"], [8, 8], update.data)
        message_id = request.message_id
        video_id = byte2int(request.video_id)
        scroll_dir = byte2int(request.scroll_dir)

        old_message = await self.message_service.select_by_id(message_id)
        chat = list(update._chat_map.values())[0]

        template = await self.video_service.videos_scroll(video_id, scroll_dir)
        await self.mybot.bot_client.edit_message(chat, old_message=old_message, template=template, link_preview=True)

    @ApiAspect(filters.Command("/favorite"))
    async def favorite(self, update: NewMessage) -> Message:
        """
        """
        template = await self.video_service.favoriate_video_init()
        message = await self.mybot.bot_client.send_message(update.chat, template=template, link_preview=True)
        return message
    
    @ApiAspect(filter=CallbackPrefix("/fr_c"))
    async def favoriate_callback(self, update: ButtonCallback) -> Message:
        """
        """
        request: CallbackBytesRequest = CallbackBytesRequest.from_byte(["random_seed", "offset"], [16, 8], update.data)
        random_seed = int(request.random_seed.decode("utf-8"))
        message_id = request.message_id
        offset = byte2int(request.offset)

        old_message = await self.message_service.select_by_id(message_id)
        chat = list(update._chat_map.values())[0]

        video = await self.video_service.favoriate_video_scroll(random_seed, offset)
        await self.mybot.bot_client.edit_message(chat, old_message=old_message, template=video, link_preview=True)
 
    @ApiAspect(filters.Command('/iamlucky'))
    async def iamlucky(self, update: NewMessage) -> Message:
        """
        I am lucky
        """
        template = await self.video_service.random_video()
        message = await self.mybot.bot_client.send_message(update.chat, template=template, link_preview=True)
        return message